package com.utopiaindustries.service;

import com.utopiaindustries.dao.ctp.*;
import com.utopiaindustries.model.ctp.*;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class DashboardService {

    private final JobCardItemDAO jobCardItemDAO;
    private final ProcessDAO processDAO;
    private final BundleDAO bundleDAO;
    private final InventoryTransactionLegDAO inventoryTransactionLegDAO;
    private final InventoryTransactionDAO inventoryTransactionDAO;
    private final JobCardDAO jobCardDAO;
    private final CryptographyService cryptographyService;
    private final MasterBundleDAO masterBundleDAO;
    private final FinishedItemDAO finishedItemDAO;
    private final StitchingOfflineItemDAO stitchingOfflineItemDAO;
    private final InventoryAccountDAO inventoryAccountDAO;
    private final PackagingItemsDAO packagingItemsDAO;


    public DashboardService(JobCardItemDAO jobCardItemDAO, ProcessDAO processDAO, BundleDAO bundleDAO, InventoryTransactionLegDAO inventoryTransactionLegDAO, InventoryTransactionDAO inventoryTransactionDAO, JobCardDAO jobCardDAO, CryptographyService cryptographyService, MasterBundleDAO masterBundleDAO, FinishedItemDAO finishedItemDAO, StitchingOfflineItemDAO stitchingOfflineItemDAO, InventoryAccountDAO inventoryAccountDAO, PackagingItemsDAO packagingItemsDAO) {
        this.jobCardItemDAO = jobCardItemDAO;
        this.processDAO = processDAO;
        this.bundleDAO = bundleDAO;
        this.inventoryTransactionLegDAO = inventoryTransactionLegDAO;
        this.inventoryTransactionDAO = inventoryTransactionDAO;
        this.jobCardDAO = jobCardDAO;
        this.cryptographyService = cryptographyService;
        this.masterBundleDAO = masterBundleDAO;
        this.finishedItemDAO = finishedItemDAO;
        this.stitchingOfflineItemDAO = stitchingOfflineItemDAO;
        this.inventoryAccountDAO = inventoryAccountDAO;
        this.packagingItemsDAO = packagingItemsDAO;
    }

    public Map<String, Float> getPhasesProgressDayWise(int lineNo) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

        String cuttingAccount = "CUTTING ACCOUNT " + lineNo;
        String stitchingAccount = "STITCHING ACCOUNT " + lineNo;
        String finishingAccount = "FINISHING ACCOUNT " + lineNo;
        String packagingAccount = "A GRADE ACCOUNT " + lineNo;

        String startDate1 = LocalDateTime.now().withHour(0).withMinute(0).withSecond(1).format(formatter);
        String endDate1 = LocalDateTime.now().withHour(23).withMinute(59).withSecond(59).format(formatter);


        HashMap<String, Float> progress = new HashMap<>();

        List<InventoryAccount> inventoryAccounts = inventoryAccountDAO.findAll();

        Map<String, Integer> inventoryAccountMap = inventoryAccounts.stream()
                .filter(e -> cuttingAccount.equals(e.getTitle()) ||
                        stitchingAccount.equals(e.getTitle()) ||
                        finishingAccount.equals(e.getTitle()) ||
                        packagingAccount.equals(e.getTitle()))
                .collect(Collectors.toMap(InventoryAccount::getTitle, e -> (int) e.getId()));



        List<Long> stitchingItemIds = inventoryTransactionLegDAO.getParentIDsOFRemainingINVENTORY( "STITCHING_OFFLINE",inventoryAccountMap.get(stitchingAccount));
        List<Long> alterationByFinishing = inventoryTransactionLegDAO.getParentIDsOFRemainingINVENTORY( "FINISHED_ITEM",inventoryAccountMap.get(stitchingAccount));

        Long approvedStitchingOfflineItems = 0L;
        Long qcReject = 0L;
        if (stitchingItemIds != null && !stitchingItemIds.isEmpty()){
            approvedStitchingOfflineItems = stitchingOfflineItemDAO.findByQCDoneDateApproved(startDate1, endDate1, "APPROVED");
            qcReject = stitchingOfflineItemDAO.findByQCDoneDate(null, endDate1, "REJECT",stitchingItemIds);
        }
        List<Long> finishedItem = inventoryTransactionLegDAO.getParentIDsOFRemainingINVENTORY( "FINISHED_ITEM",inventoryAccountMap.get(finishingAccount));

        List<Long> packagingItems = inventoryTransactionLegDAO.getParentIDsOFRemainingINVENTORY( "PACKAGING",inventoryAccountMap.get(packagingAccount));

        Long approvedFinishedItem = 0L;
        Long alterFinishedItem = 0L;
        Long bGradeFinishedItem = 0L;
        Long rejectFinishedItem = 0L;
        Long washFinishedItem = 0L;
        if(packagingItems != null && !packagingItems.isEmpty() ){
            approvedFinishedItem = finishedItemDAO.findByOperationDoneDate(startDate1, endDate1,"APPROVED");
            bGradeFinishedItem = finishedItemDAO.findByOperationDoneDate(startDate1, endDate1,"B GRADE");
            rejectFinishedItem = finishedItemDAO.findByOperationDoneDate(startDate1, endDate1,"REJECT");
            washFinishedItem = finishedItemDAO.findByOperationDoneDate(startDate1, endDate1,"WASHED");

        }

        progress.put("Stitching",(float) approvedStitchingOfflineItems +  qcReject + alterationByFinishing.size());
        progress.put("totalWips", (float) stitchingItemIds.size());
        progress.put("Alteration", (float) qcReject );

        progress.put("finishing", (float) approvedFinishedItem);
        progress.put("ALTER", (float) alterationByFinishing.size());
        progress.put("Reject",(float) rejectFinishedItem);
        progress.put("wash",(float) washFinishedItem);
        progress.put("packaging",(float) packagingItems.size() );


        return progress;
    }

    public Map<String, String> getLineDetails() {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

        String startDate1 = LocalDateTime.now().withHour(0).withMinute(0).withSecond(1).format(formatter);
        String endDate1 = LocalDateTime.now().withHour(23).withMinute(59).withSecond(59).format(formatter);
        String startDateWithHour =  LocalDateTime.now().minusHours(1).format(formatter);
        String endDateWithHour =LocalDateTime.now().format(formatter);

        HashMap<String, String> details = new HashMap<>();
        List<PackagingItems> packagingItems = packagingItemsDAO.findByDate(startDate1, endDate1);
        List<PackagingItems> packagingItemsHourly = packagingItemsDAO.findByDate(startDateWithHour, endDateWithHour);

        List<JobCard> jobCards = jobCardDAO.findAll();
        int target = 1000;
        int achieved = packagingItems.size();


        details.put("Shift Target", 1000 + " pc");
        details.put("Shift Achieved", achieved + " pc");
        details.put("Hourly Target", 1000/24 + " pc");
        details.put("Hourly Achieved", packagingItemsHourly.size() + " pc");
        details.put("Efficiency",(float)achieved / target * 100 + " %" );
        details.put("Total Induction", String.valueOf(jobCards.size()));
        details.put("Total Machine", String.valueOf(30));
        details.put("Total Worker", String.valueOf(30));
        return details;
    }
}
